# -*- coding: utf-8 -*-
from operator import attrgetter
from pyangbind.lib.yangtypes import RestrictedPrecisionDecimalType
from pyangbind.lib.yangtypes import RestrictedClassType
from pyangbind.lib.yangtypes import TypedListType
from pyangbind.lib.yangtypes import YANGBool
from pyangbind.lib.yangtypes import YANGListType
from pyangbind.lib.yangtypes import YANGDynClass
from pyangbind.lib.yangtypes import ReferenceType
from pyangbind.lib.base import PybindBase
from collections import OrderedDict
from decimal import Decimal
from bitarray import bitarray
import six

# PY3 support of some PY2 keywords (needs improved)
if six.PY3:
  import builtins as __builtin__
  long = int
elif six.PY2:
  import __builtin__

class openconfig_ospfv2_lsdb(PybindBase):
  """
  This class was auto-generated by the PythonClass plugin for PYANG
  from YANG module openconfig-ospfv2-lsdb - based on the path /openconfig-ospfv2-lsdb. Each member element of
  the container is represented as a class variable - with a specific
  YANG type.

  YANG Description: An OpenConfig model for the Open Shortest Path First (OSPF)
version 2 link-state database (LSDB)
  """
  _pyangbind_elements = {}

  

class openconfig_ospf_policy(PybindBase):
  """
  This class was auto-generated by the PythonClass plugin for PYANG
  from YANG module openconfig-ospf-policy - based on the path /openconfig-ospf-policy. Each member element of
  the container is represented as a class variable - with a specific
  YANG type.

  YANG Description: This module defines extensions to the OpenConfig policy
model to add extensions for OSPF. This module is intended
to be generic for both OSPFv2 and OSPFv3.
  """
  _pyangbind_elements = {}

  

class openconfig_ospf_types(PybindBase):
  """
  This class was auto-generated by the PythonClass plugin for PYANG
  from YANG module openconfig-ospf-types - based on the path /openconfig-ospf-types. Each member element of
  the container is represented as a class variable - with a specific
  YANG type.

  YANG Description: Type definitions for OSPF
  """
  _pyangbind_elements = {}

  

class openconfig_ospfv2_common(PybindBase):
  """
  This class was auto-generated by the PythonClass plugin for PYANG
  from YANG module openconfig-ospfv2-common - based on the path /openconfig-ospfv2-common. Each member element of
  the container is represented as a class variable - with a specific
  YANG type.

  YANG Description: This submodule provides OSPFv2 configuration and operational
state parameters that are shared across multiple contexts
  """
  _pyangbind_elements = {}

  

class openconfig_ospfv2(PybindBase):
  """
  This class was auto-generated by the PythonClass plugin for PYANG
  from YANG module openconfig-ospfv2 - based on the path /openconfig-ospfv2. Each member element of
  the container is represented as a class variable - with a specific
  YANG type.

  YANG Description: An OpenConfig model for Open Shortest Path First (OSPF)
version 2
  """
  _pyangbind_elements = {}

  

class openconfig_ospfv2_global(PybindBase):
  """
  This class was auto-generated by the PythonClass plugin for PYANG
  from YANG module openconfig-ospfv2-global - based on the path /openconfig-ospfv2-global. Each member element of
  the container is represented as a class variable - with a specific
  YANG type.

  YANG Description: This submodule provides OSPFv2 configuration and operational
state parameters that are global to a particular OSPF instance
  """
  _pyangbind_elements = {}

  

class openconfig_ospfv2_area(PybindBase):
  """
  This class was auto-generated by the PythonClass plugin for PYANG
  from YANG module openconfig-ospfv2-area - based on the path /openconfig-ospfv2-area. Each member element of
  the container is represented as a class variable - with a specific
  YANG type.

  YANG Description: This submodule provides OSPFv2 configuration and operational
state parameters that are specific to the area context
  """
  _pyangbind_elements = {}

  

class openconfig_ospfv2_area_interface(PybindBase):
  """
  This class was auto-generated by the PythonClass plugin for PYANG
  from YANG module openconfig-ospfv2-area-interface - based on the path /openconfig-ospfv2-area-interface. Each member element of
  the container is represented as a class variable - with a specific
  YANG type.

  YANG Description: This submodule provides OSPFv2 configuration and operational
state parameters that are specific to the area context
  """
  _pyangbind_elements = {}

  

class openconfig_ospfv2_lsdb(PybindBase):
  """
  This class was auto-generated by the PythonClass plugin for PYANG
  from YANG module openconfig-ospfv2-lsdb - based on the path /openconfig-ospfv2-lsdb. Each member element of
  the container is represented as a class variable - with a specific
  YANG type.

  YANG Description: An OpenConfig model for the Open Shortest Path First (OSPF)
version 2 link-state database (LSDB)
  """
  _pyangbind_elements = {}

  

class openconfig_ospfv2_common(PybindBase):
  """
  This class was auto-generated by the PythonClass plugin for PYANG
  from YANG module openconfig-ospfv2-common - based on the path /openconfig-ospfv2-common. Each member element of
  the container is represented as a class variable - with a specific
  YANG type.

  YANG Description: This submodule provides OSPFv2 configuration and operational
state parameters that are shared across multiple contexts
  """
  _pyangbind_elements = {}

  

class openconfig_ospfv2_global(PybindBase):
  """
  This class was auto-generated by the PythonClass plugin for PYANG
  from YANG module openconfig-ospfv2-global - based on the path /openconfig-ospfv2-global. Each member element of
  the container is represented as a class variable - with a specific
  YANG type.

  YANG Description: This submodule provides OSPFv2 configuration and operational
state parameters that are global to a particular OSPF instance
  """
  _pyangbind_elements = {}

  

class openconfig_ospfv2_common(PybindBase):
  """
  This class was auto-generated by the PythonClass plugin for PYANG
  from YANG module openconfig-ospfv2-common - based on the path /openconfig-ospfv2-common. Each member element of
  the container is represented as a class variable - with a specific
  YANG type.

  YANG Description: This submodule provides OSPFv2 configuration and operational
state parameters that are shared across multiple contexts
  """
  _pyangbind_elements = {}

  

class openconfig_ospfv2_area(PybindBase):
  """
  This class was auto-generated by the PythonClass plugin for PYANG
  from YANG module openconfig-ospfv2-area - based on the path /openconfig-ospfv2-area. Each member element of
  the container is represented as a class variable - with a specific
  YANG type.

  YANG Description: This submodule provides OSPFv2 configuration and operational
state parameters that are specific to the area context
  """
  _pyangbind_elements = {}

  

class openconfig_ospfv2_area_interface(PybindBase):
  """
  This class was auto-generated by the PythonClass plugin for PYANG
  from YANG module openconfig-ospfv2-area-interface - based on the path /openconfig-ospfv2-area-interface. Each member element of
  the container is represented as a class variable - with a specific
  YANG type.

  YANG Description: This submodule provides OSPFv2 configuration and operational
state parameters that are specific to the area context
  """
  _pyangbind_elements = {}

  

class openconfig_ospfv2_lsdb(PybindBase):
  """
  This class was auto-generated by the PythonClass plugin for PYANG
  from YANG module openconfig-ospfv2-lsdb - based on the path /openconfig-ospfv2-lsdb. Each member element of
  the container is represented as a class variable - with a specific
  YANG type.

  YANG Description: An OpenConfig model for the Open Shortest Path First (OSPF)
version 2 link-state database (LSDB)
  """
  _pyangbind_elements = {}

  

class openconfig_ospfv2_area_interface(PybindBase):
  """
  This class was auto-generated by the PythonClass plugin for PYANG
  from YANG module openconfig-ospfv2-area-interface - based on the path /openconfig-ospfv2-area-interface. Each member element of
  the container is represented as a class variable - with a specific
  YANG type.

  YANG Description: This submodule provides OSPFv2 configuration and operational
state parameters that are specific to the area context
  """
  _pyangbind_elements = {}

  

class openconfig_ospfv2_common(PybindBase):
  """
  This class was auto-generated by the PythonClass plugin for PYANG
  from YANG module openconfig-ospfv2-common - based on the path /openconfig-ospfv2-common. Each member element of
  the container is represented as a class variable - with a specific
  YANG type.

  YANG Description: This submodule provides OSPFv2 configuration and operational
state parameters that are shared across multiple contexts
  """
  _pyangbind_elements = {}

  

