__model_versions__ = """openconfig-icmpv6-types@2023-05-02
openconfig-packet-match@2023-03-01
openconfig-packet-match-types@2023-01-29
openconfig-icmpv4-types@2023-01-26
openconfig-acl@2023-02-06
openconfig-aft-types@2022-05-05
openconfig-aft-pf@2023-04-19 (belongs-to openconfig-aft)
openconfig-aft@2023-04-19
openconfig-aft-ipv6@2023-04-19 (belongs-to openconfig-aft)
openconfig-aft-common@2023-04-19 (belongs-to openconfig-aft)
openconfig-aft-ethernet@2023-04-19 (belongs-to openconfig-aft)
openconfig-aft-ipv4@2023-04-19 (belongs-to openconfig-aft)
openconfig-aft-mpls@2023-04-19 (belongs-to openconfig-aft)
openconfig-aft-state-synced@2023-04-19 (belongs-to openconfig-aft)
openconfig-aft-network-instance@2023-04-25
openconfig-ate-intf@2019-08-07
openconfig-ate-flow@2022-04-29
openconfig-bfd@2023-02-06
openconfig-bgp-common-multiprotocol@2023-06-27 (belongs-to openconfig-bgp)
openconfig-bgp-global@2023-06-27 (belongs-to openconfig-bgp)
openconfig-bgp-peer-group@2023-06-27 (belongs-to openconfig-bgp)
openconfig-bgp@2023-06-27
openconfig-bgp-policy@2023-03-27
openconfig-bgp-common-structure@2023-06-27 (belongs-to openconfig-bgp)
openconfig-bgp-neighbor@2023-06-27 (belongs-to openconfig-bgp)
openconfig-bgp-errors@2023-03-31 (belongs-to openconfig-bgp-types)
openconfig-bgp-common@2023-06-27 (belongs-to openconfig-bgp)
openconfig-bgp-types@2023-03-31
openconfig-catalog-types@2018-11-21
openconfig-module-catalog@2018-11-21
openconfig-defined-sets@2022-12-14
openconfig-terminal-device-properties@2022-04-26
openconfig-terminal-device-property-types@2022-03-08
openconfig-codegen-extensions@2020-10-05
openconfig-metadata@2020-08-06
openconfig-fw-high-availability@2021-06-16
openconfig-fw-link-monitoring@2021-06-16
openconfig-gribi@2021-04-06
openconfig-if-8021x@2020-01-28
openconfig-if-ip-ext@2018-11-21
openconfig-if-tunnel@2018-11-21
openconfig-if-ethernet@2023-03-10
openconfig-if-ip@2023-04-12
openconfig-if-sdn-ext@2021-03-30
openconfig-if-aggregate@2022-06-28
openconfig-if-ethernet-ext@2018-11-21
openconfig-if-poe@2018-11-21
openconfig-interfaces@2023-02-06
openconfig-isis-types@2022-02-11
openconfig-isis-routing@2023-05-01 (belongs-to openconfig-isis)
openconfig-isis-policy@2023-04-28
openconfig-isis-lsp@2023-05-01 (belongs-to openconfig-isis)
openconfig-isis-lsdb-types@2018-11-21
openconfig-isis@2023-05-01
openconfig-keychain@2022-11-05
openconfig-keychain-types@2022-03-01
openconfig-lacp@2021-07-20
openconfig-lldp@2018-11-21
openconfig-lldp-types@2018-11-21
openconfig-local-routing@2022-11-01
openconfig-macsec@2023-06-08
openconfig-macsec-types@2019-07-01
openconfig-mpls-sr@2018-11-21
openconfig-mpls-ldp@2023-02-06
openconfig-mpls-rsvp@2023-02-06
openconfig-mpls-types@2021-12-01
openconfig-mpls-te@2023-04-28 (belongs-to openconfig-mpls)
openconfig-mpls-igp@2023-04-28 (belongs-to openconfig-mpls)
openconfig-mpls@2023-04-28
openconfig-mpls-static@2023-04-28 (belongs-to openconfig-mpls)
openconfig-igmp@2023-02-03
openconfig-pim@2023-03-06
openconfig-igmp-types@2018-11-21
openconfig-pim-types@2018-11-21
openconfig-network-instance-l3@2022-11-08
openconfig-route-summary@2022-12-21
openconfig-network-instance-policy@2018-11-21
openconfig-network-instance@2023-04-25
openconfig-programming-errors@2022-10-11
openconfig-network-instance-l2@2023-04-25 (belongs-to openconfig-network-instance)
openconfig-evpn@2023-03-08
openconfig-evpn-types@2021-06-21
openconfig-network-instance-types@2021-07-14
openconfig-openflow-types@2022-05-24
openconfig-openflow@2018-11-21
openconfig-terminal-device@2021-07-29
openconfig-transport-line-connectivity@2019-06-27
openconfig-optical-attenuator@2019-07-19
openconfig-wavelength-router@2021-07-26
openconfig-transport-line-common@2019-06-03
openconfig-transport-types@2023-02-08
openconfig-channel-monitor@2019-10-24
openconfig-optical-amplifier@2019-12-06
openconfig-transport-line-protection@2018-11-21
openconfig-ospfv2-lsdb@2023-03-06 (belongs-to openconfig-ospfv2)
openconfig-ospf-policy@2018-11-21
openconfig-ospf-types@2018-11-21
openconfig-ospfv2-common@2023-03-06 (belongs-to openconfig-ospfv2)
openconfig-ospfv2@2023-03-06
openconfig-ospfv2-global@2023-03-06 (belongs-to openconfig-ospfv2)
openconfig-ospfv2-area@2023-03-06 (belongs-to openconfig-ospfv2)
openconfig-ospfv2-area-interface@2023-03-06 (belongs-to openconfig-ospfv2)
openconfig-p4rt@2022-08-19
openconfig-pcep@2023-04-25
openconfig-platform-healthz@2023-04-11
openconfig-platform-common@2023-02-13 (belongs-to openconfig-platform)
openconfig-platform-fan@2018-11-21
openconfig-platform-psu@2018-11-21
openconfig-platform-types@2023-06-27
openconfig-platform-ext@2018-11-21
openconfig-platform-fabric@2022-07-28
openconfig-platform-software@2021-06-16
openconfig-platform-port@2023-03-22
openconfig-platform-transceiver@2023-05-03
openconfig-platform-linecard@2023-02-13
openconfig-platform-controller-card@2022-07-28
openconfig-platform-integrated-circuit@2022-04-20
openconfig-platform-pipeline-counters@2023-02-03
openconfig-platform-cpu@2018-11-21
openconfig-platform@2023-02-13
openconfig-pf-interfaces@2023-04-25 (belongs-to openconfig-policy-forwarding)
openconfig-pf-srte@2019-10-15
openconfig-pf-forwarding-policies@2023-04-25 (belongs-to openconfig-policy-forwarding)
openconfig-pf-path-groups@2023-04-25 (belongs-to openconfig-policy-forwarding)
openconfig-policy-forwarding@2023-04-25
openconfig-policy-types@2022-11-08
openconfig-routing-policy@2022-05-24
openconfig-probes@2018-11-21
openconfig-probes-types@2018-11-21
openconfig-qos-elements@2023-04-25 (belongs-to openconfig-qos)
openconfig-qos-mem-mgmt@2023-04-25 (belongs-to openconfig-qos)
openconfig-qos-interfaces@2023-04-25 (belongs-to openconfig-qos)
openconfig-qos@2023-04-25
openconfig-qos-types@2018-11-21
openconfig-relay-agent@2023-02-06
openconfig-rib-bgp-ext@2019-04-25
openconfig-rib-bgp-tables@2022-12-20 (belongs-to openconfig-rib-bgp)
openconfig-rib-bgp@2022-12-20
openconfig-rib-bgp-attributes@2022-12-20 (belongs-to openconfig-rib-bgp)
openconfig-rib-bgp-shared-attributes@2022-12-20 (belongs-to openconfig-rib-bgp)
openconfig-rib-bgp-table-attributes@2022-12-20 (belongs-to openconfig-rib-bgp)
openconfig-rib-bgp-types@2019-03-14
openconfig-sampling-sflow@2022-06-21
openconfig-sampling@2022-06-21
openconfig-rsvp-sr-ext@2019-07-09
openconfig-segment-routing-types@2020-02-04
openconfig-srte-policy@2021-07-28
openconfig-segment-routing@2023-02-06
openconfig-spanning-tree-types@2021-06-16
openconfig-spanning-tree@2019-11-28
openconfig-system-utilization@2023-02-13
openconfig-system-controlplane@2023-03-03
openconfig-aaa-types@2018-11-21
openconfig-system-terminal@2018-11-21
openconfig-aaa@2022-07-29
openconfig-license@2020-04-22
openconfig-alarm-types@2018-11-21
openconfig-system-grpc@2022-04-19
openconfig-aaa-tacacs@2022-07-29 (belongs-to openconfig-aaa)
openconfig-alarms@2019-07-09
openconfig-procmon@2019-03-15
openconfig-system@2023-06-16
openconfig-messages@2018-08-13
openconfig-system-logging@2023-05-04
openconfig-aaa-radius@2022-07-29 (belongs-to openconfig-aaa)
openconfig-system-bootz@2023-06-16
openconfig-telemetry@2018-11-21
openconfig-telemetry-types@2018-11-21
openconfig-yang-types@2021-07-14
openconfig-inet-types@2023-02-06
openconfig-types@2019-04-16
openconfig-vlan@2023-02-07
openconfig-vlan-types@2022-05-24
openconfig-ap-manager@2023-05-26
openconfig-access-points@2023-06-26
openconfig-wifi-mac@2023-05-26
openconfig-ap-interfaces@2023-06-26
openconfig-wifi-phy@2023-05-26
openconfig-wifi-types@2022-05-26
"""