from oc_config_validate.testcases import config_state  # noqa
from oc_config_validate.testcases import get  # noqa
from oc_config_validate.testcases import set  # noqa
from oc_config_validate.testcases import setget  # noqa
from oc_config_validate.testcases import static_route  # noqa
from oc_config_validate.testcases import subif_ip  # noqa
from oc_config_validate.testcases import telemetry_once  # noqa
from oc_config_validate.testcases import telemetry_onchange  # noqa
from oc_config_validate.testcases import telemetry_sample  # noqa
